pub type MLDSA44 = MLDSA<MLDSA44_PK_LEN, MLDSA44_SK_LEN, MLDSA44_SIG_LEN, MLDSA44PublicKey, MLDSA44PrivateKey, MLDSA44_TAU, MLDSA44_LAMBDA, MLDSA44_GAMMA1, MLDSA44_GAMMA2, MLDSA44_k, MLDSA44_l, MLDSA44_ETA, MLDSA44_BETA, MLDSA44_OMEGA, MLDSA44_C_TILDE, MLDSA44_POLY_Z_PACKED_LEN, MLDSA44_POLY_W1_PACKED_LEN, MLDSA44_LAMBDA_over_4, MLDSA44_GAMMA1_MINUS_BETA, MLDSA44_GAMMA2_MINUS_BETA, MLDSA44_GAMMA1_MASK_LEN>;Expand description
The ML-DSA-44 algorithm.
Aliased Type
pub struct MLDSA44 { /* private fields */ }Trait Implementations
Sourceimpl Algorithm for MLDSA44
impl Algorithm for MLDSA44
Sourceconst ALG_NAME: &'static str = ML_DSA_44_NAME
const ALG_NAME: &'static str = ML_DSA_44_NAME
String name for the algorithm, used consistently across the library.
Sourceconst MAX_SECURITY_STRENGTH: SecurityStrength = SecurityStrength::_128bit
const MAX_SECURITY_STRENGTH: SecurityStrength = SecurityStrength::_128bit
Maximum security strength supported by the algorithm.
In other words, this algorithm can produce outputs up to this security strength,
but may produce outputs with lower security strength, for example, if asked to truncate.