pub type MLDSA87 = MLDSA<MLDSA87_PK_LEN, MLDSA87_SK_LEN, MLDSA87_SIG_LEN, MLDSA87PublicKey, MLDSA87PrivateKey, MLDSA87_TAU, MLDSA87_LAMBDA, MLDSA87_GAMMA1, MLDSA87_GAMMA2, MLDSA87_k, MLDSA87_l, MLDSA87_ETA, MLDSA87_BETA, MLDSA87_OMEGA, MLDSA87_C_TILDE, MLDSA87_POLY_Z_PACKED_LEN, MLDSA87_POLY_W1_PACKED_LEN, MLDSA87_LAMBDA_over_4, MLDSA87_GAMMA1_MINUS_BETA, MLDSA87_GAMMA2_MINUS_BETA, MLDSA87_GAMMA1_MASK_LEN>;Expand description
The ML-DSA-87 algorithm.
Aliased Type
pub struct MLDSA87 { /* private fields */ }Trait Implementations
Sourceimpl Algorithm for MLDSA87
impl Algorithm for MLDSA87
Sourceconst ALG_NAME: &'static str = ML_DSA_87_NAME
const ALG_NAME: &'static str = ML_DSA_87_NAME
String name for the algorithm, used consistently across the library.
Sourceconst MAX_SECURITY_STRENGTH: SecurityStrength = SecurityStrength::_256bit
const MAX_SECURITY_STRENGTH: SecurityStrength = SecurityStrength::_256bit
Maximum security strength supported by the algorithm.
In other words, this algorithm can produce outputs up to this security strength,
but may produce outputs with lower security strength, for example, if asked to truncate.